/*
Copyright (C) 1999-2007 Erwin Waterlander

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/*   History
     1.06,  Mar 20 1999
            -  All versions read both DOS and UNIX text files (skip '\r')
            -  Attached GNU General Public Licence.
     1.07,  Jan 31 2007
            -  All versions can use option -c
            -  All versions wait for keypress by default.
*/

/*
 Borland C :
   _Windows  : defined for windows 16 and 32 bit compilations
   __WIN32__ : defined for windows 32 bit compilations
 Not all other windows compilers define _Windows
*/

#ifdef __WIN32__
#define _Windows  /* Windows 16 bit */
#endif

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#ifdef __DOS__
 #include <dos.h>
#else
 #ifdef _Windows
  #include <windows.h>
 #else
  #include <unistd.h> /* unix */
 #endif
#endif
#ifdef USE_CONIO
 #include <conio.h>
#endif

#define VERSION "1.07"
#define VERSION_DATE "31 januari 2007"
#define FILENAME "verjrdag.txt"
#define MAXNAMELENGHT 1024
#if (defined(__DOS__) || defined(_Windows))
# define ALT_SW || *argv[i]=='/'
#else
# define ALT_SW
#endif

int maanden[] = {31, 31, 28, 31, 30, 31, 30, 31, 31, 30 ,31 ,30 ,31, 31};
	         /* dec,jan,feb,mrt,apr,mei,jun,jul,aug,sep,okt,nov,dec,jan */

 typedef struct opt
{
  unsigned char lang;           /* 0 = dutch, 1 = english */
  char filename[256];           /* input filename */
  unsigned char cont;           /* 0 = stop, 1 = continue */
  int days;
  int secs;
  int prev;                     /* previous days */
} my_opt;


/*****************************************************************
 * Rekent het verschil uit tussen vandaag en de ingelezen datum.
 * Een datum in de verleden tijd levert een negatief resultaat.
 * Een datum in de toekomst levert een positief resultaat.
 *****************************************************************/

int verschil(struct tm *Test, struct tm *Today)
{
	int i, t = 0, vandaag = 0;

	/* Als het nu december is en de ingelezen test-datum in januari */

	if((Test->tm_mon == 1) && (Today->tm_mon == 12))
	 {Test->tm_mon  = 13;
	   Test->tm_year--;
			}

	/* Als het nu januari is en de ingelezen test-datum in december */

	if((Test->tm_mon == 12) && (Today->tm_mon == 1))
	 {Test->tm_mon  = 0 ;
	   Test->tm_year++;
			}

	for(i=0; i < (Test->tm_mon) ;i++)
	 t += maanden[i];
	t += Test->tm_mday;

	for(i=0; i < (Today->tm_mon) ;i++)
	 vandaag += maanden[i];
	vandaag += Today->tm_mday;


  return(t - vandaag);

}

/************************************************************************/
#ifdef USE_CONIO

void printhelp( void )
{
	  textcolor(GREEN);
	  cprintf("VERJRDAG %s (%s)- Verjaardagkalender.\r\n",VERSION,VERSION_DATE);
	  cprintf("Geschreven door : Erwin Waterlander. http://www.xs4all.nl/~waterlan/\r\n");
	  cprintf("Gebruik : verjrdag [-d #] [-p #] [-c #] [<bestand>]\r\n\n");

	  cprintf("Opties: \r\n");
	  cprintf("        -L    Print software licentie.\r\n");
	  cprintf("        -d    Het aantal waarschuwingsdagen (standaard is 5).\r\n");
	  cprintf("        -p    Het aantal dagen geleden (standaard is 2).\r\n");
	  cprintf("        -c    Continueer na # seconden.\r\n\n");
	  cprintf("Voorbeeld:    verjrdag -d 4 -p 3 -c 2 verjrdag.txt\r\n\n");

	  cprintf("<bestand> is de naam van een tekstbestand waarin op elke regel\r\n");
	  cprintf("een geboortedatum staat op de volgende manier:\r\n\n");
	  cprintf("dd mm jjjj naam\r\n\n");
	  cprintf("Bijvoorbeeld:\r\n");
	  cprintf("31  1 1938 Beatrix\r\n");
	  cprintf("17  8 1968 Erwin Waterlander\r\n");

}
#else
void printhelp( void )
{
	  printf("VERJRDAG %s (%s) - Verjaardagkalender.\n",VERSION,VERSION_DATE);
	  printf("Geschreven door : Erwin Waterlander. http://www.xs4all.nl/~waterlan/\n");
	  printf("Gebruik : verjrdag [-d #] [-p #] [-c #] [<bestand>]\n\n");

	  printf("Opties: \n");
	  printf("        -L    Print software licentie.\n");
	  printf("        -d    Het aantal waarschuwingsdagen (standaard is 5).\n");
	  printf("        -p    Het aantal dagen geleden (standaard is 2).\n");
	  printf("        -c    Continueer na # seconden.\n\n");
	  printf("Voorbeeld:    verjrdag -d 4 -p 3 -c 2 verjrdag.txt\n\n");

	  printf("<bestand> is de naam van een tekstbestand waarin op elke regel\n");
	  printf("een geboortedatum staat op de volgende manier:\n\n");
	  printf("dd mm jjjj naam\n\n");
	  printf("Bijvoorbeeld:\n");
	  printf("31  1 1938 Beatrix\n");
	  printf("17  8 1968 Erwin Waterlander\n");
}

#endif

#if defined _Windows && !defined(__WIN32__)
/* WIN16 sleep function, provided by Claude G. Beaudoin */

void    YieldWindows(void)
{
        MSG             msgThing;

        while(PeekMessage(&msgThing, NULL, 0, 0, PM_REMOVE))
        {
                TranslateMessage(&msgThing);
                DispatchMessage(&msgThing);
        }

}

void    Snooze(int Seconds)
{
        time_t  CurTime, StartTime;
        HCURSOR hcurSave;

        // Change mouse pointer to hour glass
        hcurSave = SetCursor(LoadCursor(NULL, IDC_WAIT));

        // Start clock
        StartTime = time(NULL);
        for(;;)
        {
                CurTime = time(NULL);
                if(difftime(CurTime, StartTime) >= (double)Seconds) break;

                // I have to always change the cursor because the YeildWindows() changes it!

                SetCursor(LoadCursor(NULL, IDC_WAIT));
                YieldWindows();
        }

        // Return mouse pointer to original
        SetCursor(hcurSave);

}
#endif

/* Wacht een aantal seconden */

void wachten( int t)
{
#ifdef __WIN32__
	Sleep(1000 * t); /* win32 */
#else
 #ifdef _Windows
	Snooze(t); /* win16 */
 #else
	sleep(t); /* dos, unix */
 #endif
#endif
}


/*****************************************************************/
int main(short argc, char ** argv)
{

  char name[MAXNAMELENGHT];
  FILE *infile;
  struct tm *dToday;
  struct tm dTest;
  int i,j,c;
  int hit=0;
  time_t tStart, tNow;
  my_opt *opties;


      /* allocate data  */
    if ((opties = (my_opt *) malloc((unsigned long)(sizeof(my_opt)))) == NULL)
      {
	 fprintf (stderr,"Error : malloc failed\r");
	 return (0);
      }

   /* Put default parameters */

   opties->lang = 0;
   strcpy(opties->filename,FILENAME);
   opties->cont = 0;
   opties->days = 5;
   opties->secs = 0;
   opties->prev = 2;


	/* ---------------------- parse the commandline ------------*/

	for (i=1; i < argc; i++)
	{
		if (*argv[i]=='-' ALT_SW) /* is it a switch? */
			switch (argv[i][1]) {
			case 'D':
			case 'd':
			case 'C':
			case 'c':
			case 'P':
			case 'p':
				break;
			case 'L':
				printf("verjrdag %s (%s) - Verjaardagkalender\n",VERSION,VERSION_DATE);
				printf("Copyright (C) 1999-2006 Erwin Waterlander\n\n");

				printf("This program is free software; you can redistribute it and/or\n");
				printf("modify it under the terms of the GNU General Public License\n");
				printf("as published by the Free Software Foundation; either version 2\n");
				printf("of the License, or (at your option) any later version.\n\n");

				printf("This program is distributed in the hope that it will be useful,\n");
				printf("but WITHOUT ANY WARRANTY; without even the implied warranty of\n");
				printf("MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n");
				printf("GNU General Public License for more details.\n\n");

				printf("You should have received a copy of the GNU General Public License\n");
				printf("along with this program; if not, write to the Free Software\n");
				printf("Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.\n");
            free(opties);
	         return (0);
				break;
         default:               /* any switch except the above */
				printhelp();
#ifdef USE_CONIO
	    textcolor(LIGHTGRAY);
	    if (opties->cont == 0)
	    {
	      cprintf("\nDruk op <Enter> om verder te gaan . . .\r\n");
	      getch();    /* wait for keypress */
	    }
	    else {wachten(opties->secs);cprintf("\n");}
#else
	    if (opties->cont == 0)
	    {
	      printf("\nDruk op <Enter> om verder te gaan . . .\n");
	      getchar();    /* wait for keypress */
	    }
	    else {wachten(opties->secs);
	    printf("\n");}
#endif
	    free(opties);
	    return (0);

			}
		else /* Not a switch. Must be a filename or parameter. */
		{


		    if ((strcmp(argv[i-1],"-d") == 0 ) || (strcmp(argv[i-1],"-D") == 0 ))
			{
			  if ( atoi(argv[i]) != 0)
			  opties->days = atoi(argv[i]);
			 }
			 else
		    if ((strcmp(argv[i-1],"-p") == 0 ) || (strcmp(argv[i-1],"-P") == 0 ))
			{
			  if ( atoi(argv[i]) >= 0)
			  opties->prev = atoi(argv[i]);

			 }
			 else
		    if ((strcmp(argv[i-1],"-c") == 0 ) || (strcmp(argv[i-1],"-C") == 0 ))
			{
			  opties->cont = 1;
			  opties->secs = atoi(argv[i]);

			 }
			 else    /* filename */

			 {
			  strncpy(opties->filename,argv[i],255);
			 }

		}


	} /* for */


	/*--- end parsing commandline -----------*/

  tStart = time(NULL);
  dToday = localtime(&tStart);

  dToday->tm_year += 1900 ;
  dToday->tm_mon  += 1 ;

  /* schrikkeljaar ?  */
	if( (  ((dToday->tm_year % 4) == 0) && ((dToday->tm_year % 100) != 0))
	  || ((dToday->tm_year % 400) == 0)
	  )
	  maanden[2] = 29;

  if  ((infile = fopen(opties->filename,"r")) == NULL)
	{
#ifdef USE_CONIO
	  printhelp();
	  textcolor(RED);
	  cprintf("Fout: Kan bestand %s niet openen. Controleer naam en pad.\r\n", opties->filename);
	  textcolor(LIGHTGRAY);
	  if (opties->cont == 0)
	  {
	   cprintf("Druk op <Enter> om verder te gaan . . .\r\n");
	   getch();    /* wait for keypress */
	  }
	  else {wachten(opties->secs);cprintf("\n");}
#else
	  printhelp();
	  printf("Fout: Kan bestand %s niet openen. Controleer naam en pad.\n", opties->filename);
	  if (opties->cont == 0)
	  {
	   printf("\nDruk op <Enter> om verder te gaan . . .\n");
	   getchar();    /* wait for keypress */
	  }
	  else {wachten(opties->secs);printf("\n");}
#endif
	  free(opties);
	  exit(1);
	}

  *name = '\0';



  while (!feof(infile) )
  {
	if(fscanf(infile,"%d %d %d ",&dTest.tm_mday,&dTest.tm_mon,&dTest.tm_year)==3)
	{

	  /* skip spaties  */
	  while ((name[0]=getc(infile)) == ' '){};
	  j=1;
	  for(i=1; ((c=getc(infile)) != '\n') && (!feof(infile)) ;i++) /* lees naam */
		if ((i < (MAXNAMELENGHT-1)) && (c!='\r'))
		 {name[i]=c;j++;}
	  name[j]='\0';  /* afsluiten naam */


	  tNow = time(NULL);
	  if ( (tNow - tStart) > 10)
	  {
		printf("Error: Timeout\n");
		break;
	  }
	  /* reken verschil uit */

	  i = verschil(&dTest,dToday);

	  /* vandaag jarig? */
	  if (i == 0 )
		{
#ifdef USE_CONIO
		 textcolor(YELLOW + BLINK);
		 cprintf("Hoera! ");
		 textcolor(LIGHTMAGENTA);
		 cprintf("%s ", name);
		 textcolor(LIGHTBLUE);
		 cprintf("is vandaag %d jaar geworden",(dToday->tm_year-dTest.tm_year));
		 textcolor(YELLOW + BLINK);
		 cprintf("!!!\r\n");
#else
		 printf("Hoera! %s is vandaag %d jaar geworden!!!\n", name ,(dToday->tm_year-dTest.tm_year));
#endif
		 hit = 1;
		}
	  else  /* niet morgen maar wel binnen n aantal dagen jarig */
		 if (  ( i > 1)  && (i < ((opties->days)+1))  )
	     {
#ifdef USE_CONIO
	     textcolor(RED);
	     cprintf ("Let op! Over %d dagen wordt %s %d jaar.\r\n", i, name,(dToday->tm_year-dTest.tm_year));
#else
	     printf ("Let op! Over %2d dagen wordt %s %d jaar.\n", i, name,(dToday->tm_year)-(dTest.tm_year));
#endif
	     hit = 1;
	     }
		 else
	       if ( i == 1)  /* Morgen jarig */
	       {
#ifdef USE_CONIO
	       textcolor(RED);
	       cprintf ("Let op! Morgen wordt %s %d jaar.\r\n", name,(dToday->tm_year-dTest.tm_year));
#else
	       printf ("Let op! Morgen wordt %s %3d jaar.\n", name,(dToday->tm_year-dTest.tm_year));
#endif
	       hit = 1;
	       }
	       else
	         if (( i == -1)&&(opties->prev > 0))   /* Gisteren jarig */
	         {
#ifdef USE_CONIO
	         textcolor(GREEN);
	         cprintf ("Gisteren is %s %d jaar geworden.\r\n", name,(dToday->tm_year-dTest.tm_year));
#else
	         printf ("Gisteren is %s %d jaar geworden.\n", name,(dToday->tm_year-dTest.tm_year));
#endif
	         hit = 1;
	         }
	         else  /* Niet gisteren maar maximaal n dagen geleden */
	           if (  (i < -1)  && (i > -((opties->prev)+1)) )
	           {
#ifdef USE_CONIO
	           textcolor(RED);
	           cprintf ("%d dagen geleden is %s %d jaar geworden.\r\n", -i, name,(dToday->tm_year-dTest.tm_year));
#else
	           printf ("%d dagen geleden is %s %d jaar geworden.\n", -1*i, name,(dToday->tm_year)-(dTest.tm_year));
#endif
	           hit = 1;
	           }

	} /* if(fscanf(infile,"%d.... */
	else  /* fout bij inlezen datum */
	{
		 while ((c = getc(infile)) != '\n' && (!feof(infile))){};

	}  /* else */
  }   /*while (!feof(infile) ) */

fclose(infile);


  if(hit) {
#ifdef USE_CONIO

    sound(1000);
    delay(50);
    nosound();

    textcolor(LIGHTGRAY);
    if (opties->cont == 0)
    {
     cprintf("\nDruk op <Enter> om verder te gaan . . .\r\n");
     getch();    /* wait for keypress */
    }
    else {wachten(opties->secs);cprintf("\n");}
#else
    if (opties->cont == 0)
    {
     printf("\nDruk op <Enter> om verder te gaan . . .\n");
     getchar();    /* wait for keypress */
    }
    else {wachten(opties->secs);printf("\n");}
#endif
  }

 free(opties);
return(0);
}


